/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */

package friedrich
import friedrich.util._

/**
 * Basic convenience functions for Friedrich applications that are
 * configurable with string options and that run pipelines.
 */
trait FriedrichApp[T <: PhaseData] extends PhaseHelpers[T] with CmdLineOptions {
  /**
   * Initialise a new data object from the given parameters.
   */
  def initData(args: String): T = initData(args.split(" "))
  
  /**
   * Initialise a new data object from the given parameters.
   */
  def initData(args: Seq[String]): T = newData()
  
  /**
   * Create a new data object with default values.
   */
  def newData(): T
  
  def runPipeline(pipeline: String, data: T) {
    runPipeline(data, data.config.pipelines(pipeline))
  }
  
  /**
   * Run the default pipeline on a new data object.
   */
  def defaultPipeline(): T = {
    val r = newData()
    defaultPipeline(r)
    r
  }
  
  /**
   * Run the default pipeline on the given data object.
   */
  def defaultPipeline(data: T) {
    runPipeline(data, data.config.pipeline)
  }
  
  /**
   * Run the default pipeline using a new data object
   * initialised from the given arguments.
   */
  def defaultPipeline(args: String): T = {
    val r = initData(args)
    defaultPipeline(r)
    r
  }
}