/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */

package friedrich

import miniasm.util._
import miniasm.genome.Kmer

/** Base trait for phases in Friedrich applications. 
 * A sequence of phases is called a pipeline.
 */
 
trait Phase[T <: PhaseData] {
	
	val MB = 1024 * 1024
  
	def run(data: T) {
		
		var tr = Runtime.getRuntime
		if (data.config.verboseMemory) {
			println("Memory now (pre-GC) free: " + tr.freeMemory/MB + "m used: " + (tr.totalMemory - tr.freeMemory) / MB + "m max: " + tr.maxMemory/MB + " m")
			println("Doing GC...")
		}
		if (data.config.gcBetweenPhases) {
			System.gc()

			tr = Runtime.getRuntime
			if (data.config.verboseMemory) {
				println("Memory now (post-GC) free: " + tr.freeMemory/MB + "m used: " + (tr.totalMemory - tr.freeMemory) / MB + "m max: " + tr.maxMemory/MB + " m")			
				println("Entering phase: " + this)
			}
		}
		
		runImpl(data)
		
	}
	
	//Code for actually running the phase should be implemented in this method
	def runImpl(data: T): Unit

}
