/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */

package friedrich

/**
 * Tools for managing and running pipelines.
 */
trait PhaseHelpers[T <: PhaseData] {
    
    def runPipeline(data: T, phaseNames: Seq[String]): Unit = {
       var phases = List[Phase[T]]()
       
        //load the phases
        for (p <- phaseNames) {
            val c = Class.forName(p)
                val cons = c.getConstructor() //no-arg constructor must exist!
                val phase = cons.newInstance().asInstanceOf[Phase[T]]
                println("Loaded phase " + phase)
                phases ::= phase
        }
       phases = phases.reverse
       runPhases(phases, data)
    }
  
    def runPhases(phases: Seq[Phase[T]], data: T) = {
      val p = new Pipeline(phases)
      p.runPhases(data)       
    }
    
    def runPipeline(data: T, pipeline: String): Unit = {
      runPipeline(data, data.config.pipelines(pipeline))
    }
}