/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */

package friedrich.graph

trait ListHoldingNode[N] {
  var to: List[N] = Nil
  var from: List[N] = Nil
}

/**
 * This graph stores all adjacency lists in the nodes themselves.
 * Note that counting all edges is an expensive operation.
 */
trait NodeLocalGraph[N <: ListHoldingNode[N]] extends Graph[N] {

  final def implAddEdge(f: N, t: N) {
    f.to ::= t
    t.from ::= f
  }
  
  final def edgesFrom(from: N) = from.from  
  final def edgesTo(to: N) = to.to  
  override def hasEdge(from: N, to: N) = from.from.contains(to)
  final def edges = nodes.flatMap(x => x.to.map((x, _)))
  override def numEdges = nodes.foldLeft(0)(_+_.to.size)
}
