/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */

package friedrich.sql
import java.sql._

/**
 * Helper trait to get a list of results from simple SQL queries.
 */
trait SQLResults[T] {
  
  def results(query: String)(implicit conn: Connection): Iterable[T] = {
	  var results: List[T] = Nil	  
	  val stat = conn.createStatement
	  val rs = stat.executeQuery(query)
	  while (rs.next) {
		  results ::= readRecord(rs)
	  }
  	  stat.close
  	  rs.close	  
	  results
  }
  
  /**
   * Implement this method to define how each record should be read.
   */
  def readRecord(rs: ResultSet): T
  
}