/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */
package miniasm

import miniasm.util._
import miniasm.genome._
import friedrich._

/** Data being passed around between various phases
 * 
 *  TODO
 *  Support cloning (best with immutable members)
 *  Support reading and writing files
 */

class AssemblyData[T <: miniasm.genome.Kmer[T]](val nodeManifest: ClassManifest[T]) extends PhaseData {
	
	/**
	 * The graph is constructed by BuildGraphPhase and used by many subsequent phases.
	 */
	var graph: PathGraph[T] = null
	
	/**
	 * Constructed by ScanReadsPhase. Essential input to BuildGraphPhase.
	 */
	var reads: Iterable[Read[T]] = Iterable.empty
	
	/**
	 * If this flag is true, KMer sets will track, for each kmer,
	 * which reads that support it.
	 */
	var trackReads = false
		
	/**
	 * KMer repository. Populated by ScanReadsPhase.
	 */
	var kmers: KmerSet[T] = null
	
	/**
	 * Contigs that have been found at the end of the assembly.
	 * Populated by FindPathsPhase.
	 */
	var contigSet: ContigSet[T] = null
	
	//For backward compatibility. TODO: phase out
	def paths: Iterable[(Iterable[T],String,String)] = contigSet.contigs.map(_.tuple)
			
	def k = config.k.toShort

	/**
	 * Configuration flags.
	 */
    var config: AsmConfiguration = null
    
    /**
     * A factory object that knows how to construct data objects
     * such as kmers and kmer sets.
     */
    var factory: Factory[T] = null
}
