/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */
package miniasm

import miniasm.genome._

object FindRepeatsPhase {
   /** 
   * Convenience function for interactive use
   */
  def apply[T <: Kmer[T]](data: AssemblyData[T]) = (new FindRepeatsPhase[T]).run(data)
  
  /** 
   * Convenience function for interactive use
   */
  def apply[T <: Kmer[T]](data: AssemblyData[T], minTotLen: Int = 8, 
      minMotifLen: Int = 3) = (new FindRepeatsPhase[T](minTotLen, minMotifLen).run(data))
}

/**
 * Look for small repeats. Input: contigs in data.contigSet.
 * 
 * @param minMotifLen minimum length of repeated motif
 * @param minTotLen minimum length of repeat string
 */
class FindRepeatsPhase[T <: Kmer[T]](minTotLen: Int = 30, minMotifLen: Int = 5) extends AsmPhase[T] {
  
    def runImpl(data: AssemblyData[T]): Unit = { 
      println("Finding repeats...")
      
      for (c <- data.contigSet.contigs;
          (start, length, pattern) <- c.repeats(minTotLen, minMotifLen)) {
    	  println("Contig: " + c + " repeated motif: " + pattern + " start offset: " + start)    		  
      }
  } 
}


