/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */
package miniasm

import miniasm.util._
import miniasm.genome.Kmer
import friedrich.graph._

object FindTipsPhase {
  /** 
   * Convenience function for interactive use
   */
  def apply[T <: Kmer[T]](data: AssemblyData[T]) = (new FindTipsPhase[T]).run(data)
}

class FindTipsPhase[T <: Kmer[T]] extends AsmPhase[T] {

  def runImpl(data: AssemblyData[T]): Unit = { 
      println("finding tips...")
	  findTips(data.graph)
  }
  
  def findTips(g: Graph[T]): Unit = {
        var edges = 0
        var tips = 0
        var totaltips = 0
		for (n <- g.nodes; if !n.ignore) {
            edges = g.edgesTo(n).size + g.edgesFrom(n).size
            if (edges < 2) {
                val cov = n.coverage
                //println("tip found, coverage = " + cov)
                if (cov < 10) {
                    //println("ignoring " + n)
                    n.tip = true
                    if (g.edgesTo(n).size == 1) {
                        totaltips += traceTipBack(g, g.edgesTo(n).head)
                    }
                    else if (g.edgesFrom(n).size == 1) {
                        totaltips += traceTipForward(g, g.edgesFrom(n).head)
                    }
                    //print(".")
                    totaltips += tips
                }
            }
		}
        println(totaltips + " tips ignored!")
  }

   def isV(g: Graph[T], n:T) = ((g.edgesTo(n).size == 0) && (g.edgesFrom(n).size > 0)) ||
           ((g.edgesTo(n).size > 0) && (g.edgesFrom(n).size == 0))

  def traceTipBack(g: Graph[T], n: T): Int = {
//        println("tracing tip back")
        var current = n
        var count = 1
        while ((g.edgesTo(current).size + g.edgesFrom(current).size == 2) && !isV(g, current)) {
            //println("ignoring " + current)
//            print(".")
            current.tip = true
            if (g.edgesTo(current).size > 0)
                current = g.edgesTo(current).head
            count += 1
        }
        count
  }

  def traceTipForward(g: Graph[T], n: T): Int = {
//        println("tracing tip forward")
        var current = n
        var count = 1
        while ((g.edgesTo(current).size + g.edgesFrom(current).size == 2) && !isV(g, current)) {
            //println("ignoring " + current)
//            print(".")
            current.tip = true
            if (g.edgesFrom(current).size > 0)
                current = g.edgesFrom(current).head
            count += 1
        }
        count
  }

}
