/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */
package miniasm

import miniasm.util._
import miniasm.genome.Kmer
import java.io.{FileWriter, BufferedWriter}
import friedrich.graph._

object GraphToDotPhase {
  /** 
   * Convenience function for interactive use
   */
  def apply[T <: Kmer[T]](data: AssemblyData[T]) = (new GraphToDotPhase[T]).run(data)
}

/**
 * Produce a .dot graph (GraphViz format) that represents the graph in the data object.
 */
class GraphToDotPhase[T <: Kmer[T]](n: Iterable[T]) extends AsmPhase[T] {

    def this() = this(List[T]())

    //TODO: do we need to retain this state (nodes) here?
    //OK to pass as a parameter instead?
    //I'd prefer for this class to be stateless. - Johan
    var nodes = n

    /*def runImpl(list: Seq[T], g: Graph[T]): Unit = {
        val outFile = new FileWriter("graph.dot")
        outFile.write("graph mygraph {\n")
        var extra = ""
        for (n <- list) {
            for (m <- g.edgesFrom(n)) {
                if (m.ignore)
                    extra = " [color=red] "
                else extra = ""
                outFile.write(n.bpdata + n.coverage.toString + " -> " + m.bpdata + m.coverage.toString + extra + ";\n")
            }
        }
        outFile.write("}")
        outFile.close()

    }*/

  def runImpl(data: AssemblyData[T]): Unit = { 
      println("generating .dot graph...")
      if (nodes.size == 0) {
          nodes = data.graph.nodes
      }
	  graphToDot(data.graph)
  }
  
  def graphToDot(g: Graph[T]): Unit = {
        val outFile = new BufferedWriter(new FileWriter("graph.dot"))
        outFile.write("digraph mygraph {\n")
        var mextra = ""
        var nextra = ""
		for (n <- nodes) {
            nextra = ""
            if (n.tip)
                nextra = nextra + "T"
            else if (n.noise)
                nextra = nextra + "N"
            else nextra = ""
            for (m <- g.edgesFrom(n)) {
                // print n -> m
                mextra = ""
                if (m.tip)
                    mextra = mextra + "T"
                else if (m.noise)
                    mextra = mextra + "N"
                else mextra = ""
                outFile.write(n + n.coverage.toString + nextra + " -> " + m + m.coverage.toString + mextra + ";\n")
            }
		}
        outFile.write("}")
        outFile.close()
  }
}
