/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */
package miniasm.util
import scala.xml.XML
import friedrich._

/**
 * Corresponds to a set of configuration flags.
 * Configurations are read from config.xml and from command line options. 
 */
class AsmConfiguration extends Configuration { 
	var k: Int = 0 
    var minKmers: Int = 0 
    var input: String = ""     
    var maxThreads: Int = 0
    var compareAss: Option[String] = None    
    var cutoff: Option[Int] = None
    var mates: Option[String] = None    
    var trackReads: Boolean = false    
}
    
/**
 * This class reads a configuration from an XML file and adjusts it
 * by using the command line options that were supplied.
 */
class ConfigReader(args: Seq[String]) extends friedrich.ConfigReader[AsmConfiguration](args) {

	def blankConfiguration = new AsmConfiguration()
  
    override def populateConfiguration(r: AsmConfiguration) = {
      super.populateConfiguration(r)
      r.k = require(resolveInt("-k", "k"), "Supply a k with -k")
      r.minKmers = require(resolveInt("-minKmers", "minKmers"), "Use -minKmers")
      r.input = require(resolveString("-input", "input"), "Use -input")    
      r.maxThreads = resolveInt("-maxThreads", "maxThreads", 1)
      r.compareAss = resolveString("-compareAss", "compareAss")
      r.cutoff = resolveInt("-cutoff", "cutoff")
      r.mates = resolveString("-mates", "mates")          
    }
    
	override def usage() {
		println("Usage: ./assembler [-config configuration] -input (reads file) -k (k-mer size)" +
		    "\t-minKmers (min. read length) [options]" )
		println("options:")        
        println("\t-compareAss <contigs.fa>\t compares assembly with another contig file")
        println("\t-cutoff N\t hard coverage cutoff")
        println("\t-pipeline PL -threads N")        
        println("\t-mates (reads file)")
        println("Configurations are read from config.xml.")
	}
}
