/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */

package miniasm.util

import java.io.FileReader
import java.io.BufferedReader
import scala.collection.immutable.Seq
import miniasm.genome._
import miniasm.genome.bpbuffer._

/** A helper class for reading in a fasta file of contigs a kmer at a time
 * 
 * @author gabriel
 *
 */

class FastaReader(contigs: String, kmer: Short) {

    val k = kmer
    var kmers = new BPKmerSet(k)

    var fReader = new FileReader(contigs)
    var buffer = new BufferedReader(fReader)
    var current = buffer.readLine()

    var mylist: Array[BPKmer] = null
    var myiterator: Iterator[BPKmer] = null

    var inline = false
    var current_contig = ""
    var next = ""
//
//    def lookup(keys: Seq[BPKmer]): Seq[BPKmer] = {
//        // required by the Read.kmersFromRead method below
//        keys.map(kmers.findOrCreate(_))
//    }

    def hasNext = { current != null }

    def getNext(): Tuple2[String, Array[BPKmer]] = {
        // returns the next contig as a tuple: contig name and array of kmers
        var current_seq = ""
        if (hasNext) {
            current_contig = current
            current = buffer.readLine()
            while ((current != null) && (current(0) != '>')) {
                current_seq += current
                current = buffer.readLine()
            }
            mylist = kmers.processRead(current_seq, false).kmers
            (current_contig, mylist)
        }
        else {
            (null, null)
        }
    }
    
}
