/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */
package miniasm.genome

/**
 * A BPSequence is a sequence of nucleotides.
 */
trait BPSequence {
  
	/**
	 * Obtain the BP at the given index (starting at 0).
	 */
	def bp(index: Int): Char
	
	/**
	 * Obtain the size of this BP sequence.
	 */
	def size: Short
	
	/**
	 * Obtain a string using the CTAG alphabet.
	 */
	def toBPString: String
	
		
	/**
	 * Find all repeated motifs. The return values are triplets of the form (start, length, pattern).
	 * Overlapping motifs cannot be found: a given part of the underlying sequence can be assigned to one
	 * repeat only.
	 */
	def repeats(minLen: Int = 5, minMotifLen: Int = 2): Seq[(Int, Int, String)] = {
	  var r = List[(Int, Int, String)]()
	  val s = toBPString	  
	  var i = 0
	  while (i < s.size) {	  
	     s.substring(i, s.size) match {
	       case Repeat(length, motifLength, pattern) => {
	         if (length >= minLen && motifLength >= minMotifLen) {
	           r ::= (i, length, pattern)
	         }
	         i += length - 1
	       }
	       case _ => {}
	     }
	     i += 1
	  }
	r
	}
}