/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */
package miniasm.genome


/**
 * An unambiguously contiguous sequence of kmers. Belongs to a specific
 * KmerSet. 
 * 
 * @param contigSet The contig set that is associated with this contig.
 */
class Contig[T <: Kmer[_]](val contigSet: ContigSet[T], val kmers: Seq[T], 
    val stopReasonFwd: Option[String] = None, val stopReasonBack: Option[String] = None) extends BPSequence {
  /**
   * All of the reads that provided evidence for this contig.
   */
	def reads: Set[Read[T]] = Set.empty
	
	/**
	 * The kmerSet that is associated with this contig.
	 */
	def kmerSet: KmerSet[T] = contigSet.kmerSet
	
	/**
	 * Compatibility: obtain this contig as a 3-tuple
	 */
	def tuple = (kmers, stopReasonFwd.getOrElse(""), stopReasonBack.getOrElse(""))

	def subcontigOf(other: Contig[T]) = {
	//TODO  
	}
	
	def size = ((kmerSet.k - 1) + kmers.size).toShort
	def bp(index: Int) = toString().charAt(index) //TODO: inefficient! 
	
	override def toString:String = {
		val k = kmerSet.k
		kmers.head.toString + kmers.drop(1).map(_.bp(k-1).toString).foldLeft("")(_+_) 
	}
	
	def toBPString = toString

	
	
}
