/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */
package miniasm.genome

trait DNAHelpers {
  /**
   * Obtain the complement of a single nucleotide.
   */
	def charComplement(bp: Char): Char = bp match {
			case 'A' => 'T'
			case 'C' => 'G'
			case 'T' => 'A'
			case 'G' => 'C'
			case _ => throw new Exception("Error")
		}
	
	/**
	 * Obtain the complement of a string of nucleotides.
	 */
	def complement(data: String): String = {
		var i = 0
		var cs = new Array[Char](data.size)
		while(i < data.size) {
		  cs(i) = charComplement(data.charAt(i))
		  i += 1
		}
		new String(cs)
	}
	
	
	/**
	 * Obtain the reverse complement of a string of nucleotides.
	 */
	def reverseComplement(data: String): String = {
	  val s = data.size
	  var i = 0
	  var cs = new Array[Char](s)
	  while (i < s) {
	    cs(i) = charComplement(data.charAt(s - i - 1))
	    i += 1
	  }
	  new String(cs)
	}
	
		
}
