/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */
package miniasm.genome

trait Factory[T <: Kmer[T]] {
  
  /**
   * Make a new kmer set.
   */
	def makeKmerSet(k: Short): KmerSet[T]
	
	/**
	 * Make a new kmer object.
	 * This method can be used when kmer objects are looked up in the graph
	 * and so on. It shouldn't be used when kmers are scanned in from a read set.
	 * Methods in the kmer set should be used in that case.
	 * 
	 * This method is not guaranteed to be efficient and should not be used for high volume processing.
	 * In such a case, first create a new kmer set and then use its methods instead.
	 */
	def makeKmer(kmer: String): Kmer[T]			
}