/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */
package miniasm.genome
import miniasm.util._
import friedrich.graph._

/**
 * A path graph backed by a kmer set. All kmers in the set are also nodes in the graph.
 * If a partial graph is desired, SetBackedPathGraph should be used instead.
 */
class KmerSetBackedPathGraph[N <: Kmer[N]](val k: Int, kmers: KmerSet[N])
   (implicit val nodeManifest: ClassManifest[N]) 
   extends MigratingGraph[N] with PathGraph[N] {
  
	final def implAddNode(node: N) {
		//no-op, assumed already to be added
	}
	
	final def nodes(): Iterable[N] = kmers.kmers
	
	final override def hasNode(node: N) = kmers.contains(node)
	final override def edgesFrom(from: N): Iterable[N] = super.edgesFrom(ensureMine(from))			
	final override def edgesTo(to: N): Iterable[N] = super.edgesTo(ensureMine(to))
	
	/**
	 * Ensure that the given node really came from this graph's kmer set,
	 * look it up otherwise
	 */
	final def ensureMine(node: N): N = {
	  if (node.set == kmers) {
	    node
	  } else {
	    kmers.apply(node)
	  }
	}
}
