/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */
package miniasm.genome

/**
 * A repeated motif in the genome.
 */
object Repeat {    
  val MAX_MOTIF_LEN = 10
  val MIN_MOTIF_LEN = 2
  
  def motifLen(s: String): Int = {
    var i = 1
    while (i < s.size) {
      if (s.size % i == 0) {
        if (s.substring(0, i) * (s.size / i) == s) {
          return i
        }
      }
     i += 1 
    }
    s.size
  }
  
  /**
   * Does the given string start with repeats of the given pattern length?
   * Returns the number of repetitions.
   */
  def startsWithRepeats(pattern: String, haystack: String): Int = {           
    var success = true
    if(haystack.size >= pattern.size &&
       haystack.substring(0, pattern.size) == pattern) {
    	 1 + startsWithRepeats(pattern, haystack.drop(pattern.size))
    } else {
    	0
    }    
  }
  
  /**
   * Test if the input string starts with a repeated motif.
   * Return values: pattern length, motif length, pattern found.
   */
  def unapply(haystack: String): Option[(Int, Int, String)] = {    
    var len = MAX_MOTIF_LEN + 1
    
    var repeats = 0
    
    while (len > MIN_MOTIF_LEN && repeats <= 1) {
    	len -= 1
    	repeats = 0    	
    	if (motifLen(haystack.take(len)) == len) {
    		repeats = startsWithRepeats(haystack.take(len), haystack)
    	}
    }
    
    if (repeats > 1) {
      Some((repeats * len, len, haystack.take(repeats * len)))
    } else {
      None
    }
  }
}