/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */
package miniasm.genome.bpbuffer
import miniasm.genome._

import miniasm.util._

abstract class BPKmer(final var set: BPKmerSet, final val data: Array[Byte], 
    final val offset: Short) extends Kmer[BPKmer] with BPBuffer.BPBufferImpl with BitHelpers  {
	
	def continuations: Seq[BPBuffer] = continuations(bpdata)
	def continuesFrom: Seq[BPBuffer] = continuesFrom(bpdata)
	
	def continuationsInSet: Seq[BPKmer] = continuationsIn(set, bpdata)
	def continuesFromInSet: Seq[BPKmer] = continuesFromIn(set, bpdata)
	
	def bpdata: BPBuffer = this

	final override def equals(that: Any) = {
		that match {
			case other: BPKmer => {
			  if (set == other.set && canonical && other.canonical) {
			    if (other.data != data || other.offset != offset) {
			      false //fail fast
			    } else {			      
			      kmerEquals(other) //expensive deep comparison
			    }			  
			  } else {
				  kmerEquals(other) //expensive deep comparison
			  }
			}
			case _ => false
		}
	}
	
	final override def hashCode = kmerHashCode
	final def size = set.k	
}