/**
 * Part of the Friedrich bioinformatics framework.
 * Copyright (C) Gabriel Keeble-Gagnere and Johan Nystrom-Persson 2010-2012.
 * Dual GPL/MIT license. Please see the files README and LICENSE for details.
 */

package miniasm.genome.bpbuffer
import miniasm.genome._

import miniasm.genome.DNAHelpers

/**
 * Utility functions for working with the compact twobit-based kmer representation.
 */
trait BitHelpers extends DNAHelpers with BitRepresentation {
	def continuations(km: BPBuffer) = twobits.map{ km.drop(1).appendTwobit(_) }
			
	def continuesFrom(km: BPBuffer) = twobits.map { km.take(km.size - 1).prependTwobit(_) }
	
	def continuationsIn(kmerSet: BPKmerSet, km: BPBuffer) = continuations(km).filter( kmerSet.contains(_)).map { kmerSet(_) }
	def continuesFromIn(kmerSet: BPKmerSet, km: BPBuffer) = continuesFrom(km).filter( kmerSet.contains(_)).map { kmerSet(_) }
		
	//old version of the function below
//	def reverseComplement(bytes: Array[Byte], offset: Int, size: Int): Array[Byte] = {
//		stringToBytes(reverseComplement(bytesToString(bytes, offset, size)))
//	}	
	
	def reverseComplement(bytes: Array[Byte], offset: Int, size: Int): Array[Byte] = {
	  val r = Array.fill[Byte](size)(0)
	  var i = 0
	  while (i < size) {
	    var j = 0
	    var b:Byte = 0
	    while (j < 4) {
	      val mask:Byte = 0xf >> (3 - j)
	      b |= complement((bytes(size - i) & mask))	      	      	    
	      j += 1
	    }
	    r(i) = b
	    i += 1
	  }
	  r
	} 
}
